# -*- coding: utf-8 -*-
#
# bulkSetStates.py - Bulk change of archive stores states
#
# Copyright (c) 2012, 2013 MailStore Software GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# MailStore Management API connection settings
USER = "admin"
PASSWORD = "admin"
HOST = "localhost"
PORT = 8463

# Requested state of newly attached archive stores
# For possible values, see "CreateStore" command at 
# http://en.help.mailstore.com/MailStore_Server_Management_API_Commands 
state = "disabled"

################### DON'T CHANGE ANYTHING BELOW THIS LINE ###################
from mailstoreapi import ManagementAPI

# Connect to MailStore Management API
api = ManagementAPI(USER, PASSWORD, HOST, PORT)

# Walk through list of attached archive stores
for store in api.GetStoreList():
    id = store["id"]
    currentState = store["requestedState"]

    # Don't change state of active archive store
    if currentState not in (state, "current"):

        # Detach archive store or change state
        if state == "detached":
            api.DetachStore(id)
            print("[DETACHED] Archive store ID: " + id)
        else:
            api.SetStoreRequestedState(id, state)
            print("[CHANGED] Archive store ID: " + id + ", State: " + state)
