# mergeFolders.py - when many pst files have been archived, you will see one branch
#                   for each file, this scripts allows you to merge these branches
#
# Copyright (c) 2012,2013 MailStore Software GmbH
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# MailStore Management API connection settings
HOST = "localhost"
USER = "admin"
PASS = "admin"
PORT = 8463

#operation is either "Exchange" [sic!], then every folder will be merged into the exisiting exchange archive folder
#or any other name (eg "Outlook") then all folders will be merged into that folder
#operation = "Exchange"
operation = "Outlook"

#enter a regular expression here, only folders that match, will be renamed
#example:
#folders that start with with the letters "Outlook" = "^Outlook"
#any folders = "."
#regex = "."
regex = "^Outlook"

#if file is accesible, only folders of users of that file will be merged
#when file is "" all users folders will be merged
#one user per line
#file = ""
file = "users.txt"

################### DON'T CHANGE ANYTHING BELOW THIS LINE ###################
import mailstoreapi
import re
from time import sleep

def move_folder(oldfolder, newfolder):
        print("="*60) 
        print("|renaming folder {} -> {}".format(oldfolder,newfolder))
        token = api.MoveFolder(oldfolder,newfolder, False)
        status = api.GetToken(token)
        while status["progressPercentage"] < 100:      
            print("|percent done:",status["progressPercentage"])
            sleep(1)
            status = api.GetToken(token)
        else:
            print("|percent done:",status["progressPercentage"])
            print("|"+status["status"])
            for v in status["messages"]:
                print("|"+v["text"]) 

def rename():
    folders = api.GetChildFolders(maxLevels="2")["childFolders"]    
    if file != "":
        with open(file) as f:
            fileusers = [user.strip() for user in f.readlines()]
            folders = [f for f in folders if f["name"] in fileusers]
    for folder in folders:        
        print("="*60)
        print("user : {}".format(folder["name"]))
        if folder["hasChildFolders"]:
            subfolders = [childFolder["name"] for childFolder in folder["childFolders"] if foldermatch.match(childFolder["name"])]
            if operation == "Exchange":
                try:
                    targetFolder = [childFolder["name"] for childFolder in folder["childFolders"] if "exchange" in childFolder["name"].lower()][0]
                except Exception as e:
                    print("no exchange folder found {}".format(e))
                    continue
                else:
                    print("exchange folder : {}".format(targetFolder))
            else:
                targetFolder = operation
                
            try:
                subfolders.remove(targetFolder)
            except Exception:
                pass
            for subfolder in subfolders:
                move_folder(folder["name"] + "/" + subfolder, folder["name"] + "/" + targetFolder)
        else:
            print("no subfolders found")

api = mailstoreapi.ManagementAPI(USER,PASS,HOST,PORT)
foldermatch = re.compile(regex)

if __name__ == "__main__":
    rename()
    print("done")
