# updateUserNames.py - updates the usernames after an AD change
#                       reads old and new users from FILE and renames
#                       them accordingly
#
# Copyright (c) 2012,2013 MailStore Software GmbH
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

#modify this according to your installation
#Administration API has to be enabled
USER = "admin"
PASS = "admin"
HOST = "localhost"
PORT = 8463

#the file which contains old and new usernames
#mailstore takes sAMAaccountName as "primary key", so you need to enter this as newname
#you can check this, with the sync test option in MailStore Client
#one user per line, format:
#oldname=newname
#comments start with # or ;
file = "users.txt"

#enter "rename" to to rename the users according to FILE
#enter "prepare" to read current users from MailStore and prepare FILE
operation = "rename"

################### DON'T CHANGE ANYTHING BELOW THIS LINE ###################
import mailstoreapi
from time import sleep

api = mailstoreapi.ManagementAPI(USER, PASS, HOST, PORT)

def moveFolder(fromFolder, toFolder):
    print("+---------------------------------")
    print("|renaming folder {} -> {}".format(fromFolder, toFolder))
    token = api.MoveFolder(fromFolder, toFolder, False)
    status = api.GetToken(token)
    while status["progressPercentage"] < 100:      
        print("|percent done:",status["progressPercentage"])
        sleep(1)
        status = api.GetToken(token)
    else:
        print("|percent done:",status["progressPercentage"])
        print("|"+status["status"])
        for v in status["messages"]:
            print("|"+v["text"])             
    print("+---------------------------------") 

def modifyUsers():
    with open(file) as f:
        for line in [lines for lines in f.readlines() if not lines.startswith(("#",";"))]:
            oldUserName, newUserName = [l.strip() for l in line.split("=")]
            currentusers = [user["userName"] for user in api.GetUserList()]
            if (oldUserName in currentusers) and (oldUserName != newUserName):
                #rename user
                if newUserName not in currentusers:
                    print("### renaming {} to {} ".format(oldUserName, newUserName),end="")
                    try:
                        api.RenameUser(oldUserName, newUserName)
                    except Exception as e:
                        print(e)
                    else:
                        print("OK")
                        sleep(1)
                #move folder
                print("moveFolder {} to {}".format(oldUserName, newUserName))
                try:
                    moveFolder(oldUserName, newUserName)
                except Exception as e:
                    print(e)
                finally:
                    sleep(3)

                #get privileges on own folder
                try:
                    info = api.GetUserInfo(newUserName)
                except Exception as e:
                    print("{} {}".format(newUserName, e))
                    
                          
                privileges = ["read"]
                if "privilegesOnFolders" in info:
                    for f in info["privilegesOnFolders"]:
                        if f["folder"] == oldUserName:
                            privileges = f["privileges"]
                            print("privileges for user {0} on {0} were {1}".format(oldUserName,privileges))

                #granting new privileges
                print("setting privilege for user {0} on folder {0} to {1} ".format(newUserName, privileges),end="")
                try:
                    api.SetUserPrivilegesOnFolder(newUserName, newUserName, privileges)
                except Exception as e:
                    print(e)
                else:
                    print("OK")
                    sleep(1)

                #revoking old privilege
                print("setting privilege for user {} on folder {} to 'none' ".format(newUserName, oldUserName),end="")
                try:
                    api.SetUserPrivilegesOnFolder(newUserName, oldUserName, ["none"])
                except Exception as e:
                    print(e)
                else:
                    print("OK")
                    sleep(1)

def prepare():
    with open(file,mode="w") as f:
        for user in api.GetUserList():
            print("user {}".format(user))
            f.write("{}=\n".format(user["userName"])) 

if __name__ == "__main__":
    if operation == "prepare":
        prepare()
    elif operation == "rename":
        modifyUsers()
    else:
        print("operation '{}' not permitted".format(operation))
    print("done")
